
% Summary:      Linerize the model and find the optimal 
%               control variables base on exact local methode. 
% author:       Tor Anders Marvik 
% organization: Department of Chemical Engineering, NTNU, 
% contact:      torandma@stud.ntnu.no  

init_nominal;   % Initialize nominal conditoions 

% Initial point
Feed=1.35;      % Nominal feed
steam=0.03;     % Nominal steam price

% Active set
actset=[4]; % set of active constraints [V1,V2, x_A, x_B, X_C]
index_u=[par.NV+1;par.NV+2;2*par.NV+6]; % Index of MV's in x 

du=[0.00001];   % Pertubation step in manipulated variables
dd=[0.00001];   % Pertubation step in disturbances

% Linerize model 
[G1,Gd,Juu,Jud,exitflag]=linerize(...
                             index_u,actset,du,dd,Feed,steam);

% Measurement and implementation error
Wd=diag([0.01,;0.01;0.01]);% Magnitude of expected disturbance
Wn=diag(ones(82,1)*0.5);   % Meassurement error ny*ny

% Run a version of the brance and bound:    
%   - Single meassurements: b3av or b3wc 
%   - Combinations of meassurements: pb3av or pb3av  

% Running b3av and b3wc
G_comb=[G1.dTcol1du;G1.dTcol2du];  % Process model
Gd1=[Gd.dTcol1dd;Gd.dTcol2dd]; % Disturbance model
	  
[B,sset,ops,ctime,flag]=b3av(G_comb,Gd1,Wd,Wn,...
                        Juu.av,Jud.av,inf,200);   
    % An error in the algotithm causes infeasible 
    % values with loss -inf to be sorted fist in B
    % and sset: Fist feasible values given by:  
    k=1;
    while B(k)<0
        k=k+1;
    end
    B=B(k:end,:);
    sset=sset(k:end,:);    

% Use exact local methode directly -------------------- 

[H,loss_av,loss_wc,loss_av_cao,loss_wc_cao]=...
 exact_local_method(sset(1,:),G_comb,Gd1,Wd,Wn,Juu.av,Jud.av);







