function [Tcol1,Tcol2] = Temp(x,par)
%% Temp.m
% Summary:      
%  Tray temperature estimator for column 1 and column 2
%  The temperature is based on a linear relation between the
%  boiling points of the pure components. 
%
%  Boiling point assumtions 
%          Tb_A=30  *C
%          Tb_B=70  *C
%          Tb_C=100 *C
%
% author:       Tor Anders Marvik 
% organization: Department of Chemical Engineering, NTNU
% contact:      torandma@stud.ntnu.no
% input         x       : condition vektor  
%               par     : Parameter struct           
% Output        Tcol1   : Tray temperature vektor column 1
%               Tcol2   : Tray temperature vektor column 2

% Temperature function
T=@(x)100-x(:,2)*(100-70)-x(:,1)*(100-30);  

% Tray compositoin matrices
xcol1=[x(1:par.NT),...
       x(par.NT+1:2*par.NT),...
       x(2*par.NT+1:3*par.NT)];
xcol2=[x(par.NV+5:par.NV+4+par.NT),...
       x(par.NV+5+par.NT:par.NV+4+2*par.NT),...
       x(par.NV+5+2*par.NT:par.NV+4+3*par.NT)]; 

Tcol1=T(xcol1);      % Tray temperatures column 1
Tcol2=T(xcol2);      % Tray temperatures column 2


end





