%% brute_force.m     
% Summary:      Direct evalutaion of loss for a selection of 
%               variables held 
%               constant 
% author:       Tor Anders Marvik 
% organization: Department of Chemical Engineering, NTNU, Norway
% contact:      torandma@stud.ntnu.no
% requires:     matlab
% version:      1.0
% todo 1.0:     Debugging "There is allways a second bug"            
% started       Oktober 2013
%%


init_nominal            % initial values for the optimization
load('xopt_F14_pV001')  % initial guess
% ---------------- Optimize in nominal point -----------------
par.actset=[];  % No active constraints
par.flag=0;     % No control
[x_opt,fval_opt,exitflag]=fmincon(@(x) ...
             costfunction(x,price,par),x0,[],[],[],[],...
             lb,ub,@(X)nonlcon(X,par),options);
[Toptcol1,Toptcol2] = Temp(x_opt,par);    
T_opt=[Toptcol1;Toptcol2];
x0=x_opt;

% -------------  Initial loss vectors ------------------------
Loss_T=zeros(1,41);     % Loss vektor, temp from col1 const. 
Loss_T_col2=zeros(1,41);% Loss vektor, temp from col2 const.
Loss_x=zeros(1,4);      % Loss vektor, a variable in x const.
Loss_F=zeros(1,2);      % Loss vektor when a ratio with F const.
Loss_comb=zeros(1,1);   % Loss when L1/D1 constant

%--------------- Insert the disturbance-----------------------
k=1;                    % For global direct evaluation
par.F=1.4; 
par.zF=[0.4 0.2 4]'; 
par.qF=1.0;
price.pD1=1.0;

%---------------- Reoptimize---------------------------------
par.actset=[];  % no active constraints
par.flag=0;
[x_reopt,fval_reopt,exitflag]=fmincon(@(x)...
            costfunction(x,price,par),x0,[],[],[],[],...
            lb,ub,@(X)nonlcon(X,par),options);

[Treoptcol1,Treoptcol2] = Temp(x_reopt,par);    

% --------- Calculation the loss ----------------------------
%   Flag = 1: x() controlled 
%   Flag = 2: A temperature in column 1 controlled 
%   Flag = 3  x1()/x2() controlled
%   Flag = 4  x()/F() controlled 

par.flag=1;                 %Keeping variable in x constant
par.actset=[];              % No active constraints

index_x=[ par.NV+1;... % L1 % coordinate in x for  CV used
             par.NV+3;...   % D1
             1;...          % xb_A
             41];           % xd_A
      
for i=1:length(index_x)
    par.index_u=index_x(i);
    par.dof=x_opt(par.index_u);        % s.p. for the CV
    [x,fval,exitflag]=fmincon(@(x)costfunction(x,price,par),...
        x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);
    if exitflag<0           % no solutions found?
         Loss_x(k,i)=inf    
    else
         Loss_x(k,i)=fval-fval_reopt(k,:) % The loss
    end
end

par.flag=2;             % Keeping variable in T constant
par.actset=[];          % No active constraints

index_T=1:41;     
for i=[1,10,20,30,41]
    par.index_u=index_T(i);
    par.dof=T_opt(par.index_u);     % s.p. for the CV
    [x,fval,exitflag]=fmincon(@(x)costfunction(x,price,par),...
        x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);
    if exitflag<0                   % no solutions found
         Loss_T(k,i)=inf    
    else
         Loss_T(k,i)=fval-fval_reopt(k,:) % The loss
    end
end

par.flag=3;             % Keeping variable in T constant
par.actset=[];          % No active constraints
index_comb=[ par.NV+1,par.NV+3 ];   % [L1 D1]   

for i=1:size(index_comb,1)
    par.index_u=index_comb(i,:);
    par.dof=x_opt(par.index_u(1))/x_opt(par.index_u(2));  
    [x,fval,exitflag]=fmincon(@(x)costfunction(x,price,par),...
        x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);
    if exitflag<0       % no solutions found?
         Loss_comb(k,i)=inf    
    else
         Loss_comb(k,i)=fval-fval_reopt(k,:) % The loss
    end
end
             
par.flag=4;     %Keeping variable in T constant
par.actset=[];  % no active constraints
index_F=    [par.NV+3;... % D1/F   
             par.NV+1];   % L1/F

for i=1:length(index_F)
    par.index_u=index_F(i);
    par.dof=x_opt(par.index_u)/par.F;  % s.p. for the CV
    [x,fval,exitflag]=fmincon(@(x)costfunction(x,price,par),...
        x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);
    if exitflag<0       % no solutions found, infeasible?
         Loss_F(k,i)=inf 
    else
         Loss_F(k,i)=fval-fval_reopt(k,:) % The loss
    end
end

% Temperature column 2
par.flag=2;             % Keeping variable in T constant
par.actset=[];          % No active constraints

index_T=1:41;     
for i=1:41
    par.index_u=index_T(i)+41;
    par.dof=T_opt(par.index_u);      
    [x,fval,exitflag]=fmincon(@(x)costfunction(x,price,par),...
        x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);
    if exitflag<0        % no solutions found
         Loss_T_col2(k,i)=inf    
    else
         Loss_T_col2(k,i)=fval-fval_reopt(k,:) % The loss
    end
end
    
    
% Selected loss velues [Loss*1000]  
Loss_selction=[Loss_x(1,:)';Loss_T([1,10,20,30,41])';...
        Loss_F';Loss_comb';Loss_T_col2([1,10,20,30,41])']*1000

%Reoptimised cv values
CV_opt=[x_reopt(index_x);Treoptcol1([1,10,20,30,41]);...
        x_reopt(par.NV+3)/par.F;x_reopt(par.NV+1)/par.F;...
        x_reopt(par.NV+1)/x_reopt(par.NV+3);...
        Treoptcol2([1,10,20,30,41])]



    
