%% constraintplot.m
% Summary:      Finds the active contraint regions in a
%               userspesified disturbance space. Defalt is
%               Feed vs price of steam. 
%               
%               The disturbance space is found by optimise 
%               in all point i a grid of the disturace space. 
%               The active set each point is marked by a 
%               binary number.    
%                         
% author:       Tor Anders Marvik 
% organization: Department of Chemical Engineering, NTNU, Norway
% contact:      torandma@stud.ntnu.no    
% started       Oktober 2012
  
clc
clear all

%Intial values                                                                           
init_nominal                     % Initial values
load('xopt_F14_pV001')           % Load nominal optimum
x0=xopt;                         % Initial guess                                                                     

%Nominal point
par.F=1.4;
price.steam=0.01;

% Calculating optimal point
par.flag=0;                      % No control 
par.actset=[];                   % No active constraints 

[x_opt,fval_opt,exitflag]=fmincon(@(x) costfunction(...
            x,price,par),x0,[],[],[],[],lb,ub,@(X)nonlcon(...
            X, par),options);
[Toptcol1,Toptcol2] = Temp(x_opt,par);
Topt=[Toptcol1;Toptcol2];       % Optimal temperatures

% Solver options
options = optimset('TolFun',10e-6,'TolCon',10e-6,...
           'MaxFunEvals',5e3,'Display','off','Algorithm',...
           'sqp','Diagnostics','off');  

%constraints
tol=1e-6;                       % Tolerance from constraint

% Resolution on x and y axis
n_x=20;                         % Resolution on the x axsis 
n_y=20;                         % Resolution on the y axsis 

% Choose disturbance space      
x_grid=linspace(1.35,1.495,n_x);% Grid disturbance on x axis 
y_grid=linspace(0.01,0.19,n_y); % Grid disturbance on y axis 

%Solver conditions
par.flag=0;                     % No control=0, Xcont=1,...
par.actset=[];                  % No controlled active const.

%Init matrices
val=zeros(n_x*n_y,9);           % Init [bin exitflag fval,x,y]
fval_mat=zeros(n_x*n_y,2);      % Init const value matrix
x_mat=zeros(n_x*n_y,2*par.NV+8);% init x data matrix

exitflag=1;                     % Convergence flag                         
k=0;                            % Count progression
time=0;                         % Monitor time

for i=1:n_x                     % Loop x axis 
        par.F=x_grid(i);        % Variable x axis 
         
   for j=1:n_y                  % Loop Y axis 
        tic                     % start time
        k=k+1;                  % point number

        price.steam=y_grid(j);  % Variable X axis   

        %run fmincon 
        [x,fval,exitflag]=fmincon(@(x)costfunction(...
                x,price,par),x0,[],[],[],[],lb,ub,...
                @(X)nonlcon(X,par),options);
        
        % Constraint valus at optimum   
        c=[x(par.NV+2),x(2*par.NV+6),x(par.NT),...
           x(par.NV+4+2*par.NT),x(par.NV+4+2*par.NT+1)];    
        
        if exitflag == 1       % Warm start 
            x0=x;    
        end
    
        bin= binary(c,c_s,tol); % Marke active set as bin num
        
        val(k,:)=[bin,exitflag,fval,y_grid(1,j),x_grid(i)];
        
        % Monitor progression
        disp([k , val(k,:),toc])   
        time=time+toc;        
        
        %save date for other functions
        x_mat(k,:)=x';
        fval_mat(k,:)=fval;
        points(i,j,:)=[x;fval;x_grid(i);y_grid(j);exitflag];

    end 
end

% save data for other funtions
data.n_x=n_x;
data.n_y=n_y;
data.val=val;   
data.x_mat=x_mat;
data.fval_mat=x_mat;

time;                           % Total time

% Plot contourplot
figure(1)
hold on 

contour(x_grid,y_grid(1,:),reshape(val(:,1),n_x,n_y),[1],...
        'Color', rbg('Orange'),'LineWidth',2);
contour(x_grid,y_grid(1,:),reshape(val(:,2),n_x,n_y),[1],...
        'Color', rbg('Blue'),'LineWidth',1);
contour(x_grid,y_grid(1,:),reshape(val(:,3),n_x,n_y),[1],...
        'Color', rbg('Red'),'LineWidth',2);
contour(x_grid,y_grid(1,:),reshape(val(:,4),n_x,n_y),[1],...
        'Color', rbg('Purple'),'LineWidth',2);
contour(x_grid,y_grid(1,:),reshape(val(:,5),n_x,n_y),[1],...
        'Color', rbg('Green'),'LineWidth',2);


