%% controlled_line.m
% Summary:      The script calcualtes a loss when controlling
%               the system and increasing and decreasing a 
%               disturbance from nominal
% author:       Tor Anders Marvik 
% organization: Department of Chemical Engineering, NTNU, Norway
% contact:      torandma@stud.ntnu.no        

clc
clear all
                                                                           
init_nominal            % initial values

%Nominal point
par.F=1.4;
price.steam=0.01;

%calculating nominal optimum
par.flag=0;             % No control 
par.actset=[];          % No active constraints 

[x_opt,fval_opt,exitflag]=fmincon(@(x) costfunction(...
                    x,price,par),x0,[],[],[],[],lb,ub,@(...
                    X)nonlcon(X, par),options);
[Toptcol1,Toptcol2] = Temp(x_opt,par);
T_opt=[Toptcol1;Toptcol2]; % Temperature at optimum

zF=[];                  % Initiate compostition vector 
F=[];                   % Initiate feed vektor


par.flag=2; %0= no cont., 1 = cont. varaible in x, 2 = cont. T
par.actset=[1;2;4];     % Index of active constraints   
par.index_u=41+41;      % Index controlled varaible
par.dof=T_opt(par.index_u);

i=1;
exitflag=1;
while par.F>1.35 && exitflag>=0
    par.F=par.F-0.001;           % disturbance step
        
    %calculate optimum
    par.actset=[];
    par.flag=0;          % No control  
    [x_reopt(:,i),fval_reopt(i),exitflag]=fmincon(@(x)...
                costfunction(x,price,par), x0,[],[],[],[],...
                lb,ub,@(X)nonlcon(X, par),options);
     
    %calculate controlled
    par.actset=[1;2;4];  % index of active constraints   
    par.flag=2;          
    [x(:,i),fval(i),exitflag]=fmincon(@(x) ...
                costfunction(x,price,par),x0,[],[],[],[],...
                lb,ub,@(X)nonlcon(X, par),options);
    exitflag

     zF(:,i)=par.zF;
     F(i)=par.F
     steam(i)=price.steam;
     x0=x(:,i); 
     i=i+1;
end

F=fliplr(F);
zF=fliplr(zF);
x=fliplr(x);
fval=fliplr(fval);
steam=fliplr(steam);
x_reopt=fliplr(x_reopt);
fval_reopt=fliplr(fval_reopt);

%reset initial conditoins
par.dof=T_opt(par.index_u);
exitflag=1;
par.F=1.4;
par.zF=[0.4,0.2,0.4]';
price.steam=0.01;

while par.F<1.5 && exitflag>=0
    par.F=par.F+0.001;      % disturbance step
    
    %calculate optimum
    par.flag=0;             % no control  
    par.actset=[];
    [x_reopt(:,i),fval_reopt(i),exitflag]=fmincon(@(x)...
                 costfunction(x,price,par),x0,[],[],[],[],...
                 lb,ub,@(X)nonlcon(X, par),options);
     
    %calculate controlled
    par.flag=2;          
    par.actset=[1;2;4];  % index of active constraints   
    [x(:,i),fval(i),exitflag]=fmincon(@(x)...
                 costfunction(x,price,par), x0,[],[],[],[],...
                 lb,ub,@(X)nonlcon(X, par),options);

     zF(:,i)=par.zF;
     F(i)=par.F
     steam(i)=price.steam;
     x0=x(:,i); 
     i=i+1;
end


F=F(2:end-1);        %first and last element is infeasible
zF=zF(:,2:end-1);    %first and last element is infeasible
x=x(:,2:end-1);      %first and last element is infeasible
fval=fval(2:end-1);
x_reopt=x_reopt(:,2:end-1);
fval_reopt=fval_reopt(2:end-1);
steam=steam(2:end-1);
Loss=fval-fval_reopt;


%save()
