%% exact_local_methode.m
% Summary:    The script calculate H matrix and loss using
%             the analytic soultion to the exact local methode. 
%             The H matrixgives the optimal combinations of 
%             mesurement to be controlled with the measurement 
%             and implementation error considered. 
%               
% author:     Tor Anders Marvik 
% org.:       Department of Chemical Engineering, NTNU, Norway
% contact:    torandma@stud.ntnu.no     
% started     Oktober 2012
  
   
function [H,loss_av,loss_wc,loss_av_cao,loss_wc_cao]=...
        exact_local_method(sset,G_comb,Gd1,Wd,Wn,Juu,Jud)

%select a subset of G1,Gd,Juu,Jud 
G1_ss=G_comb(sset(1,:),:);
Gd_ss=Gd1(sset(1,:),:);
Wd_ss=Wd;                       % Magnitude disturbance
Wn_ss=Wn(sset(1,:),sset(1,:));  % Weighting of the noise 

ny=length(sset);                % Number of meassurements 
nd=3;                           % Number of disturbances
F_ss=-G1_ss*(Juu)^-1*Jud+Gd_ss;
Y=[F_ss*Wd_ss' Wn_ss];

H=(inv(Y*Y')*G1_ss*inv(G1_ss'*inv(Y*Y')*G1_ss)*sqrtm(Juu))';

%Loss
M=sqrtm(Juu)*inv(H*G1_ss)*H*Y;
loss_wc=0.5*svds(M,1)^2 ;     
loss_av=0.5*norm(M,'fro')^2;  

%worst case loss alternative 
G=G1_ss*inv(sqrtm(Juu));
Y2=Y*Y';
X=chol(Y2)'\G;
lambda=eig(X'*X);   
Loss_wc2=0.5./min(lambda);

%average loss alternative, uniform distribution
G=G1_ss*inv(sqrtm(Juu));
Y2=Y*Y';
X=chol(Y2)'\G;
lambda=eig(X'*X);
loss_wc_cao=0.5./min(lambda);

G2=G*G';
[R,f]=chol(G2);
X=R'\Y;
lambda=sum(sum(X.*X));
loss_av_cao=sort(lambda/(6*(size(Wd,2)+1)));

end