% Initial conditions for the optimization problem 
% defalt nominal point is F=1.4 and steam 0.01

%This script contains inital values 

%intial values
par.F=1.4;                         % Feed flow rate
par.zF=[0.4 0.2 0.4]';             % Feed composition
par.qF=1.0;                        % Feed liquid fraction
par.NC=3;                          % Number of components
par.NT=41;                         % Number of trays
par.NF=21;                         % Feed tray number
par.alpha=[2 1.5 1]';              % Relative volatility

%Prices for the cost funciton
price.pF1=1;                       % Feed price col. 1
price.pD1=1;                       % Top product price col. 1
price.pD2=2;                       % Top product price col. 2
price.pB2=1;                       % Btm product price col. 2
price.steam=0.01;                  % vapour cost

%Number of decision variables
par.NV=par.NT*par.NC;              % Number cons. decition var                               

%Define the constraint limits
V1max=4.008;                       % Max vapour flow col1
V2max=2.405;                       % Max vapour flow col2
Xd1Amin=0.95;                      % Min comp A top col1
Xd2Bmin=0.95;                      % Min comp B top col2
Xb2Cmin=0.95;                      % Max vapour flow col2


c_s=[V1max V2max Xd1Amin Xd2Bmin Xb2Cmin];  %constraints

%Initial Values
x0_col1=[   linspace(0.01,0.99,par.NT)';
            linspace(0.39,0.005,par.NT)';
            linspace(0.6,0.005,par.NT)';
            par.F/2;
            par.F/2;
            par.F/2;
            par.F/2] ;
x0_col2=[   linspace(0.00,0.00,par.NT)';
            linspace(0.01,0.99,par.NT)';
            linspace(0.99,0.01,par.NT)';    
            par.F/4;
            par.F/4;
            par.F/4;
            par.F/4] ;
        
x0=[x0_col1;x0_col2];


%Define the lower and upper bounds
lb_col1=zeros((par.NV+4),1);                
ub_col1=[ones(par.NV,1); [20 20 20 20]'];
lb_col2=zeros((par.NV+4),1);
ub_col2=[ones(par.NV,1); [20 20 20 20]'];


%Define constraints through lower and upper bounds
lb_col1(par.NT)=Xd1Amin;           % Min comp A top col1 
lb_col2(2*par.NT)=Xd2Bmin;         % Min comp B top col2
lb_col2(2*par.NT+1)=Xb2Cmin;       % Min comp C btm col2
ub_col1(par.NV+2)=V1max;           % Max vapour flow col1
ub_col2(par.NV+2)=V2max;           % Max vapour flow col2

lb=[lb_col1;lb_col2 ];             % Lower bounds
ub=[ub_col1;ub_col2 ];             % Upper bounds 


%fmincon options
options = optimset('TolFun',10e-8,'TolCon',10e-8,...
          'MaxFunEvals',1e4,'Display','off','Algorithm',...
          'sqp','Diagnostics','off');  


T=@(x)100-x(:,2)*(100-70)-x(:,1)*(100-30); % Temp. estimator 
