
function [G1 Gd,Juu,Jud,errorflag,exitflag]=linerize(index_u,...
                                        actset,du,dd,Feed,steam)

% This function calculates a linear aproximatinon of the model
% around the optimal point with fixed disturbances. The
% disturbances is set to be Feed consentration change of A
% with respect to B and consentration change of B with respect
% to C
%
% Output: 
%   G1 :  process model (dy/du)
%   Gd1:  disturbance model (dy/dd)
%   Juu:  \partial^2 J/\partial u^2
%   Jud:  \partial^2 J/\partial u\partial d
%   errorflag : 1= good, -1= Juu or Jud bad,-2= convergence error  
%   exitflag  : vektor of exitflags with all the function evaluations

% Input: 
%   index_u :  Index manipuleted variables
%   actset  :  Index of avtive contrains 
%   du      :  Magnitude of step in the manipuleted variables  
%   dd      :  Magnitude of step in the disturbances

%Example use

%Internal variables
% Matrices used to create the the derivatives
%             
% fval_up=
%  d= none        d1+dd1      d2 +dd2       dn+ddn
% [ fval_opt    fval        fval          fval  ]   u=none
% | fval                                        |   u1+du1
% |                                             |   u2+du2                  
% [ fval                                  fval  ]   un+dun
%             
% fval_down= 
%   d= none        d1-dd1      d2-dd2        dn-ddn    
% [ fval_opt    fval        fval          fval  ]   u=none
% | fval                                        |   u1-du1
% |                                             |   u2-du2                  
% [ fval                                  fval  ]   un-dun

%               
% x_up =  
%   du1   du2 ... dun
% [ x1(u1+du1)     x1(u2+du2)  ...   x1(u2+du2) ]
% [ x2(u1+du1)     x2(u2+du2)  ...   x2(u2+du2) ]
% |    :               :              :         |
% [ xn(u1+du1)     xn(u2+du2)  ...   xn(u2+du2) ]

% x_up =         
% [ x1(u1-du1)     x1(u2-du2)  ...   x1(u2-du2) ]
% [ x2(u1-du1)     x2(u2+du2)  ...   x2(u2-du2) ]
% |    :               :              :         |
% [ xn(u1-du1)     xn(u2-du2)  ...   xn(u2-du2) ]
%
% Tcol1_up and Tcol2_down is similar to x_up 

if length(actset)+length(index_u)>4
    error('myApp:argChk', 'The system is overspesified')
end

if length(actset)+length(index_u)<4
    error('myApp:argChk', 'The system is underspesified')
end

% Setting intial conditions
init_nominal        %Initial conditions and solver options        
par.F=Feed;         %Nominal feed flow
price.steam=steam;  %Nominal steam price
par.actset=actset;  %index of active constraints
par.index_u=index_u;%index of controlled variables in x vector

nu=length(index_u); % number of inputs
nd=3;               % number of disturbances

% Prepering matrixes
fval_up=zeros(nu+1,nd+1);
fval_down=zeros(nu+1,nd+1);
x_up=zeros(length(x0),nu);
x_down=zeros(length(x0),nu);
Tcol1_up=zeros(41,nu);
Tcol2_up=zeros(41,nu);
Tcol1_down=zeros(41,nu);
Tcol2_down=zeros(41,nu);
errorflag=1;

p=1;                 % index to monitor exitflag

%Optimal point
par.flag=0;          % No control
par.actset=[];       % No active constraints

[x_opt,fval_opt,exitflag]=fmincon(@(x)costfunction(x,price,par),...     
         x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);        
[Tcol1_opt,Tcol2_opt] = Temp(x_opt,par);
x0=x_opt;
fval_up(1,1)=fval_opt;

par.actset=actset;   % Set active constraints
par.flag=1;         % Controlled variable in the x vektor
for i = 1:nu
%u+du    
par.dof=x_opt(index_u);     % setpoint MV's       
par.dof(i)=par.dof(i)+du;    % pertuation in ui positive  
        % No disturbance
        par.F=Feed;
        par.zF=[0.4; 0.2 ;0.4];
        [x_up(:,i),fval_up(i+1,1),exitflag(p)]=fmincon(@(x)...
                    costfunction(x,price,par),x0,[],[],[],[],...
                    lb,ub,@(X)nonlcon(X,par),options);        
        [Tcol1_up(:,i),Tcol2_up(:,i)] = Temp(x_up(:,i),par);
        p=p+1; % monitor exitflag
    for j=1:nu
               par.dof=x_opt(index_u);  
               par.dof(i)=par.dof(i)+du; % pertuation in ui positive  
               par.dof(j)=par.dof(j)+du;
               [x,fval_u_up(i,j),exitflag(p)]=fmincon(@(x)...
                   costfunction(x,price,par),x0,[],[],[],[],...
                   lb,ub,@(X)nonlcon(X,par),options);        
               p=p+1; % monitor exitflag
    end
par.dof=x_opt(index_u);  
par.dof(i)=par.dof(i)+du;  
    
    % positive disturbance in feed
    par.F=Feed+dd;
    par.zF=[0.4; 0.2 ;0.4];
    [x,fval_up(i+1,2),exitflag(p)]=fmincon(@(x)costfunction(x,price,par),...     
             x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);           
         p=p+1; % monitor exitflag   
         
    % disturbance in zF_A
  	par.F=Feed;
    par.zF=[0.4+dd; 0.2-dd ;0.4];
    [x,fval_up(i+1,3),exitflag(p)]=fmincon(@(x)costfunction(x,price,par),...     
             x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);        
        p=p+1; % monitor exitflag
        
    % disturbance in zF_B
  	par.F=Feed;
    par.zF=[0.4; 0.2+dd ;0.4-dd];
    [x,fval_up(i+1,4),exitflag(p)]=fmincon(@(x)costfunction(x,price,par),...     
             x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);   
       p=p+1; % monitor exitflag
       
%u-du
par.dof=x_opt(index_u);  
par.dof(i)=par.dof(i)-du;    % pertuation in u negative
    par.F=Feed;
    par.zF=[0.4; 0.2;0.4];
    [x_down(:,i),fval_down(i+1,1),exitflag(p)]=fmincon(@(x)costfunction(x,price,par),...     
             x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);        
    [Tcol1_down(:,i),Tcol2_down(:,i)] = Temp(x_down(:,i),par);
    p=p+1; % monitor exitflag     
    for j=1:nu
               par.dof=x_opt(index_u);  
               par.dof(i)=par.dof(i)-du;    % pertuation in ui positive  
               par.dof(j)=par.dof(j)-du;
               [x,fval_u_down(i,j),exitflag(p)]=fmincon(@(x)costfunction(x,price,par),...     
                x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);        
               p=p+1; % monitor exitflag
    end

par.dof=x_opt(index_u);  
par.dof(i)=par.dof(i)-du;    % pertuation in u negative
    % negative disturbance in feed 
    par.F=Feed-dd;
    par.zF=[0.4; 0.2 ;0.4];
    [x,fval_down(i+1,2),exitflag(p)]=fmincon(@(x)costfunction(x,price,par),...     
             x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);           
        p=p+1; % monitor exitflag     
    % disturbance in zF_A
  	par.F=Feed;
    par.zF=[0.4-dd; 0.2+dd ;0.4];
    [x,fval_down(i+1,3),exitflag(p)]=fmincon(@(x)costfunction(x,price,par),...     
             x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);        
        p=p+1; % monitor exitflag     
    % disturbance in zF_B
  	par.F=Feed;
    par.zF=[0.4; 0.2-dd ;0.4+dd];
    [x,fval_down(i+1,4),exitflag(p)]=fmincon(@(x)costfunction(x,price,par),...     
             x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);   
    p=p+1; % monitor exitflag    
    
end


%Disturbances no pertubation in u
par.dof=x_opt(index_u);

% Disturbance in feed up
par.F=Feed+dd;
par.zF=[0.4; 0.2 ;0.4];
[x_d_up(:,1),fval_up(1,2),exitflag(p)]=fmincon(@(x)costfunction(x,price,par),...     
   x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);        
[Tcol1_d_up(:,1),Tcol2_d_up(:,1)] = Temp(x_d_up(:,1),par);
p=p+1; % monitor exitflag   

% Disturbance in feed down
par.F=Feed-dd;
par.zF=[0.4; 0.2 ;0.4];
[x_d_down(:,1),fval_down(1,2),exitflag(p)]=fmincon(@(x)costfunction(x,price,par),...     
             x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);  
[Tcol1_d_down(:,1),Tcol2_d_down(:,1)] = Temp(x_d_down(:,1),par);
p=p+1; % monitor exitflag   

% Disturbance in zF_A up
par.F=Feed;
par.zF=[0.4+dd; 0.2-dd ;0.4];
[x_d_up(:,2),fval_up(1,3),exitflag(p)]=fmincon(@(x)costfunction(x,price,par),...     
             x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);       
[Tcol1_d_up(:,2),Tcol2_d_up(:,2)] = Temp(x_d_up(:,2),par);
p=p+1; % monitor exitflag   

% Disturbance in zF_A down
par.F=Feed;
par.zF=[0.4-dd; 0.2+dd ;0.4];
[x_d_down(:,2),fval_down(1,3),exitflag(p)]=fmincon(@(x)costfunction(x,price,par),...     
             x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);   
[Tcol1_d_down(:,2),Tcol2_d_down(:,2)] = Temp(x_d_down(:,2),par);
p=p+1; % monitor exitflag   

% Disturbance in zF_B up
par.F=Feed;
par.zF=[0.4; 0.2+dd ;0.4-dd];
[x_d_up(:,3),fval_up(1,4),exitflag(p)]=fmincon(@(x)costfunction(x,price,par),...     
             x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);  
[Tcol1_d_up(:,3),Tcol2_d_up(:,3)] = Temp(x_d_up(:,3),par);
p=p+1; % monitor exitflag   

% Disturbance in zF_B down
par.F=Feed;
par.zF=[0.4; 0.2-dd ;0.4+dd];
[x_d_down(:,3),fval_down(1,4),exitflag(p)]=fmincon(@(x)costfunction(x,price,par),...     
             x0,[],[],[],[],lb,ub,@(X)nonlcon(X,par),options);   
[Tcol1_d_down(:,3),Tcol2_d_down(:,3)] = Temp(x_d_down(:,3),par);
 
p=p+1; % monitor exitflag   
%======================== Gu = dy/du =====================================
%G= [dx1/du1 dx1/du2 ... dx1/dun
%    dx2/du1 dx2/du2 ... dx2/dun
%       :       :           : 
%    dxm/du1 dxm/du2 ... dxm/dun]

G1.dxdu=(x_up-x_down)./(2*du);    
G1.dTcol1du=(Tcol1_up-Tcol1_down)./(2*du);
G1.dTcol2du=(Tcol2_up-Tcol2_down)./(2*du);    

G1.dTcol1du_f=(Tcol1_up-Tcol1_opt*ones(1,nu))./(du);
G1.dTcol2du_f=(Tcol2_up-Tcol2_opt*ones(1,nu))./(du);    

G1.dTcol1du_b=(-Tcol1_down+Tcol1_opt*ones(1,nu))./(du);
G1.dTcol2du_b=(-Tcol2_down+Tcol2_opt*ones(1,nu))./(du);  
       
%========================= Gd = d/dd =====================================

%Gd=dx/dd   = [dx1/dF dx2/dz1 ... dx1/dz2
%              dx2/dF dx2/dz1 ... dx2/dz2
%                :       :           : 
%              dxm/F dxm/dz1 ... dxm/dz2]

Gd.dxdd=(x_d_up-x_d_down)./(2*dd);
Gd.dTcol1dd=(Tcol1_d_up-Tcol1_d_down)./(2*dd);
Gd.dTcol2dd=(Tcol2_d_up-Tcol2_d_down)./(2*dd);

Gd.dTcol1dd_f=(Tcol1_d_up-Tcol1_opt*ones(1,nd))./(dd);
Gd.dTcol2dd_f=(Tcol2_d_up-Tcol2_opt*ones(1,nd))./(dd);

Gd.dTcol1dd_b=(Tcol1_opt*ones(1,nd)-Tcol1_d_down)./(dd);   
Gd.dTcol2dd_b=(Tcol2_opt*ones(1,nd)-Tcol2_d_down)./(dd);


%========================== Juu = d2J/dudu =============================== 

for i=1:nu
     for j=1:nu
    Juu.up(i,j)=(fval_u_up(i,j)-fval_up(j+1,1)-fval_up(i+1,1)+fval_opt)./(du^2);
    Juu.down(i,j)=(fval_u_down(i,j)-fval_down(j+1,1)-fval_down(i+1,1)+fval_opt)./(du^2);
     end
end

Juu.av=(Juu.up+Juu.down)./2;  % Average forward and backwards differace

% Error: The forward and backward differance is significantly different
if norm(Juu.up)-norm(Juu.down)> 1
    errorflag=-1;
end
%========================== Jud = d2J/dudd =============================== 

for i=1:nu
     for j=1:nd
         % Central differance: does not work
         Jud.up(i,j)=(fval_up(i+1,j+1)-fval_up(1,j+1)-fval_up(i+1,1)+fval_opt)./(du*dd);
         Jud.down(i,j)=(fval_down(i+1,j+1)-fval_down(1,j+1)-fval_down(i+1,1)+fval_opt)./(du*dd);
     end
end
 

Jud.av=(Jud.up+Jud.down)./2;  % Average forward and backwards differace

% Error: The forward and backward differance is significantly different
if norm(Jud.up)-norm(Jud.down)> 1
    errorflag=-1;
end

% Error: One of the avaluations failed to converge
if min(exitflag)<0
    errorflag=-2;
end
    
end

    