
% The script acquires all the neassesary paramteters and finds the optimal
% control structure using the branch and bound algoritm developed by Yi Cao

% Calculate the figures 5.3.12 - 5.3.14 in the thesis.


init_nominal;               % Initialize nominal conditoions 


% Nominal_point

Feed=1.4;                   % Nominal feed
steam=0.01;                 % Nominal steam price

% Active set
actset=[1;2;4];     % set of active constraints from [V1,V2, x_A, x_B, X_C]

index_u=[par.NV+1]; % Index of MV's in x vector
du=[0.00001];       % Pertubation step in MV's
dd=[0.00001];       % Pertubation step in disturbances


% Linerize model 
[G1,Gd,Juu,Jud,exitflag]=linerize(index_u,actset,du,dd,Feed,steam);

Wd=diag([0.01,;0.01;0.01]); % Magnitude of expected disturbance nd*nd
Wn=diag(ones(41,1)*0.5);    % Meassurement error ny*ny


    figure(4)
    hold on
    %When controling a temperature in column 1 
    % All disturbances included
    [B,sset,ops,ctime,flag]=b3wc(G1.dTcol1du,Gd.dTcol1dd(:,:),...
        diag([0.01,0.01,0.01]),Wn,Juu.av,Jud.av(1,:),inf,41);
        ssetB=[sset,B];
        [Y,I]=sort(ssetB(:,1));
        ssetB=ssetB(I,:); %use the column indices from sort() to sort all columns of A.
        plot(ssetB(:,1),ssetB(:,2),'k')

    %Only disturbance in F
    [B,sset,ops,ctime,flag]=b3wc(G1.dTcol1du,Gd.dTcol1dd(:,1),diag([0.01]),Wn,Juu.av,Jud.av(1,1),inf,41);
        ssetB=[sset,B];
        [Y,I]=sort(ssetB(:,1));
        ssetB=ssetB(I,:); %use the column indices from sort() to sort all columns of A.
        plot(ssetB(:,1),ssetB(:,2),'b.-')

    %Only disturbance in zF_A
    [B,sset,ops,ctime,flag]=b3wc(G1.dTcol1du,Gd.dTcol1dd(:,2),diag([0.01]),Wn,Juu.av,Jud.av(1,2),inf,41);
        ssetB=[sset,B];
        [Y,I]=sort(ssetB(:,1));
        ssetB=ssetB(I,:); %use the column indices from sort() to sort all columns of A.
        plot(ssetB(:,1),ssetB(:,2),'g-.')

    %Only disturbance in zF_B
    [B,sset,ops,ctime,flag]=b3wc(G1.dTcol1du,Gd.dTcol1dd(:,3),diag([0.01]),Wn,Juu.av,Jud.av(1,3),inf,41);
        ssetB=[sset,B];

    [Y,I]=sort(ssetB(:,1));
        ssetB=ssetB(I,:); %use the column indices from sort() to sort all columns of A.
        plot(ssetB(:,1),ssetB(:,2),'r--')

    axis([0,41,0,3e-3])
    legend('Combination of all disturbances','Disturbance in feed','Disturbance in zF_A on the expense of zF_B','Disturbance in zF_B on the expense of zF_C')
    xlabel('Contolled tray in column 1')
    ylabel('Loss [$/s]')
    title('Loss controlling a tray temperature from column 1')


    figure(6)
    hold on
    %When controling a temperature in column 2 
    % All disturbances included
        [B,sset,ops,ctime,flag]=b3wc(G1.dTcol2du,Gd.dTcol2dd(:,:),...
        diag([0.01,0.01,0.01]),Wn,Juu.av,Jud.av(1,:),inf,41);
        ssetB=[sset,B];
        sset 
        B
        [Y,I]=sort(ssetB(:,1));
        ssetB=ssetB(I,:); %use the column indices from sort() to sort all columns of A.
        plot(ssetB(:,1),ssetB(:,2),'k')

    %Only disturbance in F
    [B,sset,ops,ctime,flag]=b3wc(G1.dTcol2du,Gd.dTcol2dd(:,1),diag([0.01]),Wn,Juu.av,Jud.av(1,1),inf,41);
        ssetB=[sset,B];
        [Y,I]=sort(ssetB(:,1));
        ssetB=ssetB(I,:); %use the column indices from sort() to sort all columns of A.
        plot(ssetB(:,1),ssetB(:,2),'b.-')

    %Only disturbance in zF_A
    [B,sset,ops,ctime,flag]=b3wc(G1.dTcol2du,Gd.dTcol2dd(:,2),diag([0.01]),Wn,Juu.av,Jud.av(1,2),inf,41);
        ssetB=[sset,B];
        [Y,I]=sort(ssetB(:,1));
        ssetB=ssetB(I,:); %use the column indices from sort() to sort all columns of A.
        plot(ssetB(:,1),ssetB(:,2),'g-.')

    %Only disturbance in zF_B
    [B,sset,ops,ctime,flag]=b3wc(G1.dTcol2du,Gd.dTcol2dd(:,3),diag([0.01]),Wn,Juu.av,Jud.av(1,3),inf,41);
        ssetB=[sset,B];

    [Y,I]=sort(ssetB(:,1));
        ssetB=ssetB(I,:); %use the column indices from sort() to sort all columns of A.
        plot(ssetB(:,1),ssetB(:,2),'r--')

    axis([0,41,0,3e-3])
    legend('Combination of all disturbances','Disturbance in feed flow','Disturbance in zF_A on the expense of zF_B','Disturbance in zF_B on the expense of zF_C')
    xlabel('Contolled tray column 2')
    ylabel('Loss [$/s]')
    title('Loss controlling a tray temperature from column 2')

    
% Combination of meassurements
figure(1)
hold on
avcol1.B=zeros(41,1);
avcol2.B=zeros(41,1);
avcol1.sset=zeros(41,41);   
avcol2.sset=zeros(41,41);   

ctime=zeros(10,1);
for i =1:10
    i
   %[avcol1.B(i,1),avcol1.sset(i,1:i),ops,avcol1.ctime(i,1),flag]=...
    %   pb3av(G1.dTcol1du,Gd.dTcol1dd(:,:),diag([0.01,0.01,0.01]),...
    %   Wn,Juu.av,Jud.av(1,:),i,inf,1);
   
  %  avcol1.ctime(i,1)
    
   % [avcol2.B(i,1),avcol2.sset(i,1:i),ops,avcol2.ctime(i,1),flag]=...
    %     pb3av(G1.dTcol2du,Gd.dTcol2dd(:,:),diag([0.01,0.01,0.01]),...
    %    Wn,Juu.av,Jud.av(1,:),i,inf,1);
   % avcol2.ctime(i,1)
    
  [wccol1.B(i,1),wccol1.sset(i,1:i),ops,wccol1.ctime(i,1),flag]=...
      pb3wc(G1.dTcol1du,Gd.dTcol1dd(:,:),diag([0.01,0.01,0.01]),...
      Wn,Juu.av,Jud.av(1,:),i,inf,1);
  
   wccol1.ctime(i,1)
    
    [wccol2.B(i,1),wccol2.sset(i,1:i),ops,wccol2.ctime(i,1),flag]=...
        pb3wc(G1.dTcol2du,Gd.dTcol2dd(:,:),diag([0.01,0.01,0.01]),...
        Wn,Juu.av,Jud.av(1,:),i,inf,1);
     wccol2.ctime(i,1)

end

figure(1)
hold on
plot(1:10,wccol1.B(1:10),'k')
plot(1:10,wccol2.B(1:10),'b--')

title('Controlling combinations of temperatures')
xlabel('Number of controlled variables')
ylabel('Loss [$/s]')
legend('Temperatures from column 1 only','Temperatures from column 2 only')

        

 